@@ -91,6 +91,12 @@ def main(args=None, function=None):  # pylint: disable=too-many-statements
         parents=[debug],
         formatter_class=common.WideHelpFormatter,
     )
+    sub.add_argument(
+        '-f',
+        '--force',
+        action='store_true',
+        help="create config file even if parent exists",
+    )
 
     # Install parser
     info = "get the specified versions of all dependencies"
@@ -105,6 +111,13 @@ def main(args=None, function=None):  # pylint: disable=too-many-statements
     sub.add_argument(
         '-e', '--fetch', action='store_true', help="always fetch the latest branches"
     )
+    sub.add_argument(
+        '-n',
+        '--no-defaults',
+        help='override default groups and install all dependencies if none specified',
+        action='store_true',
+        dest='no_defaults',
+    )
 
     # Update parser
     info = "update dependencies to the latest versions"
@@ -235,6 +248,9 @@ def _get_command(function, namespace):  # pylint: disable=too-many-statements
 
     if namespace.command == 'init':
         function = commands.init
+        kwargs.update(
+            force=namespace.force,
+        )
 
     elif namespace.command in ['install', 'update']:
         function = getattr(commands, namespace.command)
@@ -248,9 +264,15 @@ def _get_command(function, namespace):  # pylint: disable=too-many-statements
             skip_changes=namespace.skip_changes,
         )
         if namespace.command == 'install':
-            kwargs.update(fetch=namespace.fetch)
+            kwargs.update(
+                fetch=namespace.fetch,
+                skip_default_group=namespace.no_defaults,
+            )
         if namespace.command == 'update':
-            kwargs.update(recurse=namespace.recurse, lock=namespace.lock)
+            kwargs.update(
+                recurse=namespace.recurse,
+                lock=namespace.lock,
+            )
 
     elif namespace.command == 'list':
         function = commands.display
@@ -263,7 +285,9 @@ def _get_command(function, namespace):  # pylint: disable=too-many-statements
     elif namespace.command == 'lock':
         function = getattr(commands, namespace.command)
         args = namespace.name
-        kwargs.update(root=namespace.root)
+        kwargs.update(
+            root=namespace.root,
+        )
 
     elif namespace.command == 'uninstall':
         function = commands.delete
@@ -276,7 +300,9 @@ def _get_command(function, namespace):  # pylint: disable=too-many-statements
     elif namespace.command == 'show':
         function = commands.show
         args = namespace.name
-        kwargs.update(root=namespace.root)
+        kwargs.update(
+            root=namespace.root,
+        )
         if namespace.config:
             args.append('__config__')
         if namespace.log:
@@ -284,7 +310,9 @@ def _get_command(function, namespace):  # pylint: disable=too-many-statements
 
     elif namespace.command == 'edit':
         function = commands.edit
-        kwargs.update(root=namespace.root)
+        kwargs.update(
+            root=namespace.root,
+        )
 
     return function, args, kwargs
 
@@ -322,7 +350,6 @@ def _run_command(function, args, kwargs):
 
 
 def _show_error(exception):
-    # TODO: require level=, evaluate all calls to dedent()
     common.dedent(0)
     common.newline()
     common.show(str(exception), color='error')
