@@ -113,16 +113,19 @@ def valid():
         return False
 
     log.debug("Checking for a valid git top level...")
-    gittoplevel = git('rev-parse', '--show-toplevel', _show=False)
-    currentdir = pwd(_show=False)
+    gittoplevel = os.path.normpath(
+        os.path.normcase(git('rev-parse', '--show-toplevel', _show=False)[0])
+    )
 
+    currentdir = os.path.normpath(os.path.normcase(pwd(_show=False)))
     status = False
-    if gittoplevel[0] == currentdir:
+
+    if gittoplevel == currentdir:
         status = True
     else:
         log.debug(
             "git top level: %s != current working directory: %s",
-            gittoplevel[0],
+            gittoplevel,
             currentdir,
         )
         status = False
@@ -162,7 +165,9 @@ def changes(type, include_untracked=False, display_status=True, _show=False):
         git('diff-index', '--quiet', 'HEAD', _show=_show)
 
         # Check for untracked files
-        lines = git('ls-files', '--others', '--exclude-standard', _show=_show)
+        lines = git(
+            'ls-files', '--others', '--exclude-standard', _show=_show, _stream=False
+        )
 
     except ShellError:
         status = True
@@ -172,7 +177,7 @@ def changes(type, include_untracked=False, display_status=True, _show=False):
 
     if status and display_status:
         with suppress(ShellError):
-            lines = git('status', _show=True)
+            lines = git('status', _show=True, _stream=False)
             common.show(*lines, color='git_changes')
 
     return status
@@ -207,7 +212,7 @@ def update(
         git('clean', '--force', '-d', '-x', _show=False)
 
     rev = _get_sha_from_rev(rev)
-    git('checkout', '--force', rev)
+    git('checkout', '--force', rev, _stream=False)
     git('branch', '--set-upstream-to', 'origin/' + rev, **hide)
 
     if fetch:
@@ -232,7 +237,7 @@ def get_hash(type, _show=False):
 
     assert type == 'git'
 
-    return git('rev-parse', 'HEAD', _show=_show)[0]
+    return git('rev-parse', 'HEAD', _show=_show, _stream=False)[0]
 
 
 def get_tag():
