@@ -1,42 +1,54 @@
 import os
-from typing import List
+import sys
+from typing import List, Optional
 
 import log
-import yorm
-from yorm.types import SortedList, String
+from datafiles import datafile, field
 
 from .. import common, exceptions, shell
+from ..decorators import preserve_cwd
 from .group import Group
 from .source import Source
 
 
-@yorm.attr(location=String)
-@yorm.attr(sources=SortedList.of_type(Source))
-@yorm.attr(sources_locked=SortedList.of_type(Source))
-@yorm.attr(groups=SortedList.of_type(Group))
-@yorm.sync("{self.root}/{self.filename}", auto_save=False)
-class Config(yorm.ModelMixin):
+@datafile("{self.root}/{self.filename}", defaults=True, manual=True)
+class Config:
     """Specifies all dependencies for a project."""
 
-    LOG = "gitman.log"
+    root: Optional[str] = None
+    filename: str = "gitman.yml"
 
-    def __init__(self, root=None, filename="gitman.yml", location="gitman_sources"):
-        super().__init__()
-        self.root = root or os.getcwd()
-        self.filename = filename
-        self.location = location
-        self.sources: List[Source] = []
-        self.sources_locked: List[Source] = []
-        self.groups: List[Group] = []
+    location: str = "gitman_sources"
+    sources: List[Source] = field(default_factory=list)
+    sources_locked: List[Source] = field(default_factory=list)
+    default_group: str = field(default_factory=str)
+    groups: List[Group] = field(default_factory=list)
 
-    def _on_post_load(self):
-        for source in self.sources:
-            source._on_post_load()  # pylint: disable=protected-access
+    def __post_init__(self):
+        if self.root is None:
+            self.root = os.getcwd()
 
-        for source in self.sources_locked:
-            source._on_post_load()  # pylint: disable=protected-access
+    @property
+    def config_path(self) -> str:
+        """Get the full path to the config file."""
+        assert self.root
+        return os.path.normpath(os.path.join(self.root, self.filename))
 
-        # check for conflicts between source names and group names
+    path = config_path
+
+    @property
+    def log_path(self) -> str:
+        """Get the full path to the log file."""
+        return os.path.normpath(os.path.join(self.location_path, "gitman.log"))
+
+    @property
+    def location_path(self) -> str:
+        """Get the full path to the dependency storage location."""
+        assert self.root
+        return os.path.normpath(os.path.join(self.root, self.location))
+
+    def validate(self):
+        """Check for conflicts between source names and group names."""
         for source in self.sources:
             for group in self.groups:
                 if source.name == group.name:
@@ -46,23 +58,6 @@ class Config(yorm.ModelMixin):
                     ).format(source.name)
                     raise exceptions.InvalidConfig(msg)
 
-    @property
-    def config_path(self):
-        """Get the full path to the config file."""
-        return os.path.normpath(os.path.join(self.root, self.filename))
-
-    path = config_path
-
-    @property
-    def log_path(self):
-        """Get the full path to the log file."""
-        return os.path.normpath(os.path.join(self.location_path, self.LOG))
-
-    @property
-    def location_path(self):
-        """Get the full path to the dependency storage location."""
-        return os.path.normpath(os.path.join(self.root, self.location))
-
     def get_path(self, name=None):
         """Get the full path to a dependency or internal file."""
         base = self.location_path
@@ -85,6 +80,7 @@ class Config(yorm.ModelMixin):
         fetch=False,
         clean=True,
         skip_changes=False,
+        skip_default_group=False,
     ):  # pylint: disable=too-many-locals
         """Download or update the specified dependencies."""
         if depth == 0:
@@ -92,7 +88,9 @@ class Config(yorm.ModelMixin):
             return 0
 
         sources = self._get_sources(use_locked=False if update else None)
-        sources_filter = self._get_sources_filter(*names, sources=sources)
+        sources_filter = self._get_sources_filter(
+            *names, sources=sources, skip_default_group=skip_default_group
+        )
 
         if not os.path.isdir(self.location_path):
             shell.mkdir(self.location_path)
@@ -116,6 +114,7 @@ class Config(yorm.ModelMixin):
                 skip_changes=skip_changes,
             )
             source.create_link(self.root, force=force)
+            source.create_links(self.root, force=force)
             common.newline()
             count += 1
 
@@ -130,6 +129,7 @@ class Config(yorm.ModelMixin):
                     fetch=fetch,
                     clean=clean,
                     skip_changes=skip_changes,
+                    skip_default_group=skip_default_group,
                 )
                 common.dedent()
 
@@ -142,14 +142,17 @@ class Config(yorm.ModelMixin):
 
         return count
 
-    def run_scripts(self, *names, depth=None, force=False):
+    @preserve_cwd
+    def run_scripts(self, *names, depth=None, force=False, show_shell_stdout=False):
         """Run scripts for the specified dependencies."""
         if depth == 0:
             log.info("Skipped directory: %s", self.location_path)
             return 0
 
         sources = self._get_sources()
-        sources_filter = self._get_sources_filter(*names, sources=sources)
+        sources_filter = self._get_sources_filter(
+            *names, sources=sources, skip_default_group=False
+        )
 
         shell.cd(self.location_path)
         common.newline()
@@ -158,17 +161,20 @@ class Config(yorm.ModelMixin):
         count = 0
         for source in sources:
             if source.name in sources_filter:
-                source.run_scripts(force=force)
-                count += 1
+                shell.cd(source.name)
 
                 config = load_config(search=False)
                 if config:
                     common.indent()
-                    count += config.run_scripts(
-                        depth=None if depth is None else max(0, depth - 1), force=force
-                    )
+                    remaining_depth = None if depth is None else max(0, depth - 1)
+                    if remaining_depth:
+                        common.newline()
+                    count += config.run_scripts(depth=remaining_depth, force=force)
                     common.dedent()
 
+                source.run_scripts(force=force, show_shell_stdout=show_shell_stdout)
+                count += 1
+
                 shell.cd(self.location_path, _show=False)
 
         common.dedent()
@@ -178,7 +184,9 @@ class Config(yorm.ModelMixin):
     def lock_dependencies(self, *names, obey_existing=True, skip_changes=False):
         """Lock down the immediate dependency versions."""
         sources = self._get_sources(use_locked=obey_existing).copy()
-        sources_filter = self._get_sources_filter(*names, sources=sources)
+        sources_filter = self._get_sources_filter(
+            *names, sources=sources, skip_default_group=False
+        )
 
         shell.cd(self.location_path)
         common.newline()
@@ -204,7 +212,7 @@ class Config(yorm.ModelMixin):
             shell.cd(self.location_path, _show=False)
 
         if count:
-            self.save()
+            self.datafile.save()
 
         common.dedent()
 
@@ -240,6 +248,7 @@ class Config(yorm.ModelMixin):
 
         for source in self.sources:
 
+            assert source.name
             yield os.path.join(self.location_path, source.name)
 
             shell.cd(self.location_path, _show=False)
@@ -278,6 +287,7 @@ class Config(yorm.ModelMixin):
 
     def log(self, message="", *args):
         """Append a message to the log file."""
+        os.makedirs(self.location_path, exist_ok=True)
         with open(self.log_path, 'a') as outfile:
             outfile.write(message.format(*args) + '\n')
 
@@ -308,28 +318,31 @@ class Config(yorm.ModelMixin):
 
         return sources + extras
 
-    def _get_sources_filter(self, *names, sources):
+    def _get_sources_filter(self, *names, sources, skip_default_group):
         """Get filtered sublist of sources."""
-        sources_filter = None
+        names_list = list(names)
 
-        groups_filter = [group for group in self.groups if group.name in list(names)]
+        if not names_list and not skip_default_group:
+            names_list.append(self.default_group)
 
-        if groups_filter:
-            sources_filter = [
-                members for group in groups_filter for members in group.members
-            ]
-        else:
-            sources_filter = list(names) if names else [s.name for s in sources]
+        # Add sources from groups
+        groups_filter = [group for group in self.groups if group.name in names_list]
+        sources_filter = [member for group in groups_filter for member in group.members]
 
-        return sources_filter
+        # Add independent sources
+        sources_filter.extend(
+            [source.name for source in sources if source.name in names_list]
+        )
+
+        if not sources_filter:
+            sources_filter = [source.name for source in sources]
+
+        return list(set(sources_filter))
 
 
 def load_config(start=None, *, search=True):
     """Load the config for the current project."""
-    if start:
-        start = os.path.abspath(start)
-    else:
-        start = os.getcwd()
+    start = os.path.abspath(start) if start else _resolve_current_directory()
 
     if search:
         log.debug("Searching for config...")
@@ -341,7 +354,7 @@ def load_config(start=None, *, search=True):
         for filename in os.listdir(path):
             if _valid_filename(filename):
                 config = Config(path, filename)
-                config._on_post_load()  # pylint: disable=protected-access
+                config.validate()
                 log.debug("Found config: %s", config.path)
                 return config
 
@@ -358,8 +371,18 @@ def load_config(start=None, *, search=True):
     return None
 
 
+def _resolve_current_directory():
+    start = os.getcwd()
+    if sys.version_info < (3, 8) and os.name == "nt":
+        log.warn("Python 3.8+ is required to resolve virtual drives on Windows")
+    else:
+        start = os.path.realpath(start)
+        os.chdir(start)
+    return start
+
+
 def _valid_filename(filename):
     name, ext = os.path.splitext(filename.lower())
     if name.startswith('.'):
         name = name[1:]
-    return name in ['gitman', 'gdm'] and ext in ['.yml', '.yaml']
+    return name in {'gitman', 'gdm'} and ext in {'.yml', '.yaml'}
