@@ -1,52 +1,48 @@
 import os
+import warnings
+from dataclasses import dataclass, field
+from typing import List, Optional
 
 import log
-import yorm
-from yorm.types import AttributeDictionary, List, NullableString, String
 
 from .. import common, exceptions, git, shell
 
 
-@yorm.attr(name=String)
-@yorm.attr(type=String)
-@yorm.attr(repo=String)
-@yorm.attr(sparse_paths=List.of_type(String))
-@yorm.attr(rev=String)
-@yorm.attr(link=NullableString)
-@yorm.attr(scripts=List.of_type(String))
-class Source(AttributeDictionary):
+@dataclass
+class Link:
+    source: str = ''
+    target: str = ''
+
+
+@dataclass
+class Source:
     """A dictionary of `git` and `ln` arguments."""
 
+    repo: str = ''
+    name: Optional[str] = None
+    rev: str = 'main'
+
+    type: str = 'git'
+    sparse_paths: List[str] = field(default_factory=list)
+    link: Optional[str] = None
+    links: List[Link] = field(default_factory=list)
+
+    scripts: List[str] = field(default_factory=list)
+
     DIRTY = '<dirty>'
     UNKNOWN = '<unknown>'
 
-    def __init__(
-        self,
-        type,
-        repo,
-        name=None,
-        rev='master',
-        link=None,
-        scripts=None,
-        sparse_paths=None,
-    ):
-
-        super().__init__()
-        self.type = type or 'git'
-        self.repo = repo
-        self.name = self._infer_name(repo) if name is None else name
-        self.rev = rev
-        self.link = link
-        self.scripts = scripts or []
-        self.sparse_paths = sparse_paths or []
-
-        for key in ['name', 'repo', 'rev']:
-            if not self[key]:
-                msg = "'{}' required for {}".format(key, repr(self))
-                raise exceptions.InvalidConfig(msg)
-
-    def _on_post_load(self):
+    def __post_init__(self):
+        if self.name is None:
+            self.name = self.repo.split('/')[-1].split('.')[0]
+        else:
+            self.name = str(self.name)
         self.type = self.type or 'git'
+        if self.link:
+            warnings.warn(
+                "'link' will be replaced with 'links' in version 3.0",
+                DeprecationWarning,
+            )
 
     def __repr__(self):
         return "<source {}>".format(self)
@@ -80,7 +76,14 @@ class Source(AttributeDictionary):
         log.info("Updating source files...")
 
         # Clone the repository if needed
-        if not os.path.exists(self.name):
+        assert self.name
+        valid_checkout_dir = False
+        if os.path.isdir(self.name):
+            valid_checkout_dir = len(os.listdir(self.name)) == 0
+        else:
+            valid_checkout_dir = True
+
+        if valid_checkout_dir:
             git.clone(
                 self.type,
                 self.repo,
@@ -148,37 +151,35 @@ class Source(AttributeDictionary):
             self.type, self.repo, self.name, fetch=fetch, clean=clean, rev=self.rev
         )
 
+    def create_links(self, root, force=False):
+        """Create links from the source to target directory."""
+        if not self.links:
+            return
+
+        for link in self.links:
+            target = os.path.join(root, os.path.normpath(link.target))
+            relpath = os.path.relpath(os.getcwd(), os.path.dirname(target))
+            source = os.path.join(relpath, os.path.normpath(link.source))
+            create_sym_link(source, target, force)
+
     def create_link(self, root, force=False):
         """Create a link from the target name to the current directory."""
         if not self.link:
             return
 
-        log.info("Creating a symbolic link...")
-
-        target = os.path.join(root, self.link)
+        target = os.path.join(root, os.path.normpath(self.link))
         source = os.path.relpath(os.getcwd(), os.path.dirname(target))
+        create_sym_link(source, target, force)
 
-        if os.path.islink(target):
-            os.remove(target)
-        elif os.path.exists(target):
-            if force:
-                shell.rm(target)
-            else:
-                msg = "Preexisting link location at {}".format(target)
-                raise exceptions.UncommittedChanges(msg)
-
-        shell.ln(source, target)
-
-    def run_scripts(self, force=False):
+    def run_scripts(self, force=False, show_shell_stdout=False):
         log.info("Running install scripts...")
 
         # Enter the working tree
-        shell.cd(self.name)
         if not git.valid():
             raise self._invalid_repository
 
         # Check for scripts
-        if not self.scripts:
+        if not self.scripts or not self.scripts[0]:
             common.show("(no scripts to run)", color='shell_info')
             common.newline()
             return
@@ -186,21 +187,23 @@ class Source(AttributeDictionary):
         # Run all scripts
         for script in self.scripts:
             try:
-                lines = shell.call(script, _shell=True)
+                shell.call(script, _shell=True, _stream=show_shell_stdout)
             except exceptions.ShellError as exc:
-                common.show(*exc.output, color='shell_error')
+                if show_shell_stdout:
+                    common.show('(script returned an error)', color='shell_error')
+                else:
+                    common.show(*exc.output, color='shell_error')
                 cmd = exc.program
                 if force:
                     log.debug("Ignored error from call to '%s'", cmd)
                 else:
                     msg = "Command '{}' failed in {}".format(cmd, os.getcwd())
                     raise exceptions.ScriptFailure(msg)
-            else:
-                common.show(*lines, color='shell_output')
         common.newline()
 
     def identify(self, allow_dirty=True, allow_missing=True, skip_changes=False):
         """Get the path and current repository URL and hash."""
+        assert self.name
         if os.path.isdir(self.name):
 
             shell.cd(self.name)
@@ -257,18 +260,19 @@ class Source(AttributeDictionary):
             return None
 
         source = self.__class__(
-            self.type,
-            self.repo,
-            self.name,
-            rev,
-            self.link,
-            self.scripts,
-            self.sparse_paths,
+            type=self.type,
+            repo=self.repo,
+            name=self.name,
+            rev=rev,
+            link=self.link,
+            scripts=self.scripts,
+            sparse_paths=self.sparse_paths,
         )
         return source
 
     @property
     def _invalid_repository(self):
+        assert self.name
         path = os.path.join(os.getcwd(), self.name)
         msg = """
 
@@ -279,8 +283,17 @@ class Source(AttributeDictionary):
         )
         return exceptions.InvalidRepository(msg)
 
-    @staticmethod
-    def _infer_name(repo):
-        filename = repo.split('/')[-1]
-        name = filename.split('.')[0]
-        return name
+
+def create_sym_link(source, target, force):
+    log.info("Creating a symbolic link...")
+
+    if os.path.islink(target):
+        os.remove(target)
+    elif os.path.exists(target):
+        if force:
+            shell.rm(target)
+        else:
+            msg = "Preexisting link location at {}".format(target)
+            raise exceptions.UncommittedChanges(msg)
+
+    shell.ln(source, target)
